module Evaluation.FunctionIO
  ( evalFunctionIO
  ) where

import Prelude hiding (String, FilePath)

import Types.Values
import Types (FunctionIO(..), Value(..), toValue, makeImage)

import qualified Data.Text.IO as T
import qualified Codec.Picture as CP
import System.Directory (listDirectory, doesDirectoryExist, createDirectory, copyFile)

evalFunctionIO :: FunctionIO -> Value -> IO Value
evalFunctionIO f x = case (f, x) of
  (ListDirectory, String (StringWrapper s)) ->
    (List . map (toValue . StringWrapper)) <$> listDirectory s

  (IsDirectory, String (StringWrapper s)) ->
    Bool <$> doesDirectoryExist s

  (ReadTextFile, String (StringWrapper s)) ->
    Text <$> T.readFile s

  (OpenImage, String (StringWrapper s)) -> do
    imageOrig <- CP.readImage s
    case imageOrig of
      Left e -> error ("unexpected error: " ++ e)
      Right image -> pure $ makeImage $ CP.convertRGB8 image

  (SaveImage, Tuple (Image (ImageWrapper image), String (StringWrapper s))) -> do
    CP.saveJpgImage 90 s $ CP.ImageRGB8 image
    pure Empty

  (SaveTextFile, Tuple (Text t, String (StringWrapper s))) -> do
    T.writeFile s t
    pure Empty

  (CopyFile, Tuple (String (StringWrapper source), String (StringWrapper target))) -> do
    copyFile source target
    pure Empty

  (MakeDir, String (StringWrapper s)) -> do
    createDirectory s
    pure Empty

  _ ->
    error ("unexpected combination of function and argument type; got function " ++ show f ++ " with argument " ++ show x)
